;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "consult-gh" "20260625.257"
  "Consulting GitHub Client."
  '((emacs         "29.4")
    (consult       "2.0")
    (markdown-mode "2.6")
    (ox-gfm        "1.0")
    (yaml          "1.2.0"))
  :url "https://github.com/armindarvish/consult-gh"
  :commit "7042619362cd47314d3d0f706fa9cac5bf07950c"
  :revdesc "7042619362cd"
  :keywords '("convenience" "matching" "tools" "vc"))
